// nifticonverter.js: script to manage the user interface nifticonverter.ui and to communicate with the NIfTI converter plugin.
// This bundle consists of *.js, *.ui and *.dylib/*.dll/*.so files, place in /Documents/BVQXExtensions/Plugins_32 or 64 (Mac)/ folder.
// Functionality: Converts between NIfTI and BrainVoyager file formats. 
// Usage: In the BrainVoyager menu, select 'Plugins' > NIfTI converter
// Latest update: 16-02-2012
// Hester Breman, Brain Innovation B.V. 

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ DECLARATIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
var scriptObj 					= new Object; 
scriptObj.scriptNameForUser 	= "NIfTI converter"; 
scriptObj.dialogFileName 		= "nifticonverter.ui"; 
scriptObj.dialogAccessName      = "niftiDlg"; 

// global variables
var command = "Unknown";
var incompleteinfo = false;
var hasposfile = false;
var hasniftifile = false;
var hastrffile = false;
var openInBVQX = false;
var doBatch = false;

// file type IDs correspond to plugin converter's IDs
const FILE_TYPE_UNKNOWN = 0;
const NIFTI_FILE_TYPE = 1; 
const NIFTI_PAIR_FILE_TYPE = 2;
const ANALYZE75_FILE_TYPE = 3;
const NIFTI_IMG_TYPE = 4;
const BV_TYPE_VMR = 20;
const BV_TYPE_FMR = 21;
const BV_TYPE_AMR = 22;
const BV_TYPE_VTC = 24;
const BV_TYPE_VOI = 27;
const BV_TYPE_DMR = 29;
const BV_TYPE_VMP = 201;
const BV_TYPE_VMP_AR = 202;
const BV_TYPE_VMP_NR = 203;
const FILE_TYPE_TXT = 401;
const BV_TYPE_MAP = 200;
const BV_TYPE_STC = 23;
const BV_TYPE_DWI = 30;
const BV_TYPE_VDW = 31;
const BV_TYPE_GLM = 205;
const BV_TYPE_AVA = 206;
const BV_TYPE_PRT = 208;
const BV_TYPE_TRF = 301;
const BV_TYPE_POS = 302;
// submenu options
const PRINT_TO_LOG = 1;
const PRINT_TO_FILE = 2;
const PRINT_TO_LOG_AND_FILE = 3;
const PRINT_HIST = 4;
// 	enum ConvMenuOptions
const CONVERT_SINGLE_FILE = 1;
const CONVERT_BATCH = 2;
const CONVERT_REGISTERED = 3;
const CONVERT_REGISTERED_BATCH = 4;
// enum POSMENUOPTIONS
const POS_MATRIX_FROM_FILE = 1;
const POS_ADD_TO_FILE = 2;
const POS_BVQXPOS_FROM_NIIPOS = 3; // not active yet
const POS_COMPUTE_COORDS_OF_POINTS = 4; // not active yet
// enum TRFMENUOPTIONS
const TRF_INVERT_MAT = 1;
const TRF_APPLY_TRF_MAN = 2;
const TRF_APPLY_TRF_FILE = 3;
const TRF_COMPUTE_BBOX = 4;

// ~~~ nicknames for children ~~~
// global tab
var optConvertButton, optInfoButton, optPosButton, optManipButton, optTrfButton, optBatchButton, startButton;
var niftiNameLabel, getNiftiFileButton;
// conversion tab
var niiFromButton, hdrFromButton, vmrFromButton, fmrFromButton, dmrFromButton, vtcFromButton, vmpFromButton, voiFromButton; 
var niiToButton, hdrToButton, vmrToButton, fmrToButton, dmrToButton, vtcToButton, vmpToButton; 
var transformBox, getBatchConvButton, getNIfTIConvButton, transformBox, flipBox, batchBox, batchFileLabel, openInBVQXBox, zeropadBox, intensityBox, vointerpBox, resetBox;
var chbSelectVols, spbFromVol, spbToVol;
// info tab
var niiPrintButton, hdrPrintButton, vmrPrintButton, fmrPrintButton, dmrPrintButton, vtcPrintButton, vmpPrintButton, voiPrintButton; 
var getniftiInfoButton, destBothButton, destFileButton, destLogButton; 
// positioning tab
var getPosFileButton, createPosFileButton, posToImgButton, printPosMatButton, calcPosButton, p00, p01, p02, p03, p10, p11, p12, p13, p20, p21, p22, p23, p30, p31, p32, p33;
var nxBox, nyBox, nzBox, dxBox, dyBox, dzBox;
// header manipulation tab
var optCopyButton, optFldManipButton; 
var leQformCode, leQb, leQc, leQd, leQoffsetX, leQoffsetY, leQoffsetZ, leSformCode, leSrow00, leSrow01, leSrow02,leSrow03, leSrow11 ,leSrow12,leSrow13, leSrow21, leSrow22, leSrow23;
var rbtnCopyPos,rbtnCopyAll, rbtnNiiType, rbtnHdrType, rbtnNiiTarget, rbtnHdrTarget, lblOtherNIfTIfile, btnOtherNIfTIfile;
var leQFac, lblNx, lblNy, lblNz, lblDx, lblDy, lblDz, calcBVposButton;
// transformation tab
var getNIfTIFileToTrfButton, invertTrfButton,applyManTrfButton,applyBVQXTrfButton, getTrfFileTypeBox, transformGrBox, t00, t01, t02, t03, t10, t11, t12, t13, t20, t21, t22, t23, t30, t31, t32, t33;
var comboboxListenerButton; // hidden button to listen to filetype changes 
 // batch tab
var writeBatchFileButton;



//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ GENERIC FUNCTIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  



scriptObj.errorfree = function() {

    var errornr = BrainVoyagerQX.GetPluginIntParameter("ErrorNr");
    if (errornr == 0) return true;
    else return false;
}

scriptObj.checkerrorstatus = function() {

    var errornr = BrainVoyagerQX.GetPluginIntParameter("ErrorNr");
    if (errornr < 0) {
            var errormsg = BrainVoyagerQX.GetPluginStringParameter("ErrorMsg");
            BrainVoyagerQX.PrintToLog(errormsg);
            cancel = true;
    } 
    return errornr;
}

scriptObj.start = function() {
    
    var success = true;   
    // check if status is prepared 
    if (optConvertButton.checked == 1) {
        this.prepareConversion();
    } else if (optInfoButton.checked == 1) {   
        success = this.prepareCollectInfo();
    } else if (optPosButton.checked == 1) {    
        this.preparePosFuncs();
    } else if (optManipButton.checked == 1) {
        this.prepareHdrManipFuncs();
    } else if (optTrfButton.checked == 1) {    
        this.prepareTrfFuncs();
    } else if (optBatchButton.checked == 1) { 
        this.prepareBatchFuncs();
    } else {
        BrainVoyagerQX.PrintToLog("Could not find which option was checked.");
        incompleteinfo = true;
    }   
    
    // then start plugin
    if ((!incompleteinfo) && (success)) {
        BrainVoyagerQX.PrintToLog("Send start signal to plugin...");
        BrainVoyagerQX.SetPluginStringParameter("Command", "Start");    
        BrainVoyagerQX.ExecutePlugin(); 
    
        // ask parameters from plugin
        if ((this.errorfree()) && (!doBatch) && (openInBVQX)) {
            var convertedname = BrainVoyagerQX.GetPluginStringParameter("ConvertedName");
            BrainVoyagerQX.PrintToLog("Opening " + convertedname + "...");
            var bvqxdoc = BrainVoyagerQX.OpenDocument(convertedname.toString());
        }
    } else {
        BrainVoyagerQX.PrintToLog("Cannot start function: incomplete information. Please check whether all parameters have been collected.");
    }   
    
    // now close the dialog
    this.niftiDlg.close(); 
}

scriptObj.getNiftiFile = function() {

    var filetypeid = FILE_TYPE_UNKNOWN;
    if (optConvertButton.checked == 1) {
        filetypeid = this.getConvertFromFileId();
    } else if (optInfoButton.checked == 1) {
        filetypeid = this.getInfoFileId();
    } else if (optTrfButton.checked == 1) {    
        filetypeid = this.getFiletypeToTransform();
    } else if (optManipButton.checked == 1) {
        filetypeid = this.getManipulatedFileType();
    } else {
        BrainVoyagerQX.PrintToLog("Source of signal to get NIfTI file not recognised.");
    }
                  
    if (filetypeid != FILE_TYPE_UNKNOWN) {          
        BrainVoyagerQX.SetPluginStringParameter("Command", "GetFile");    
        BrainVoyagerQX.SetPluginIntParameter("FileID", filetypeid);
        BrainVoyagerQX.ExecutePlugin(); 
        var niftifilename = BrainVoyagerQX.GetPluginStringParameter("NiftiFileName");
        niftiNameLabel.text = niftifilename;
        hasniftifile = true;  
                
        // show the parameters in the `Header' tab
        var matrix = this.getMatrixParametersFromPlugin();
        this.setSformMatrix(matrix);
        leQformCode.text    = BrainVoyagerQX.GetPluginIntParameter("QFormCode");
        leSformCode.text    = BrainVoyagerQX.GetPluginIntParameter("SFormCode");
        leQb.text           = BrainVoyagerQX.GetPluginFloatParameter("Qb");
        leQc.text           = BrainVoyagerQX.GetPluginFloatParameter("Qc");
        leQd.text           = BrainVoyagerQX.GetPluginFloatParameter("Qd");
        leQoffsetX.text     = BrainVoyagerQX.GetPluginFloatParameter("QoffsetX");
        leQoffsetY.text     = BrainVoyagerQX.GetPluginFloatParameter("QoffsetY");
        leQoffsetZ.text     = BrainVoyagerQX.GetPluginFloatParameter("QoffsetZ");
        leQFac.text         = BrainVoyagerQX.GetPluginFloatParameter("qfac");
        lblNx.text          = BrainVoyagerQX.GetPluginIntParameter("nx");
        lblNy.text          = BrainVoyagerQX.GetPluginIntParameter("ny");
        lblNz.text          = BrainVoyagerQX.GetPluginIntParameter("nz");
        lblDx.text          = BrainVoyagerQX.GetPluginFloatParameter("dx");
        lblDy.text          = BrainVoyagerQX.GetPluginFloatParameter("dy");
        lblDz.text          = BrainVoyagerQX.GetPluginFloatParameter("dz");  
    }    
}

scriptObj.isniftifile = function(filetypeid) {
    
    if ((filetypeid == NIFTI_FILE_TYPE) || (filetypeid == NIFTI_PAIR_FILE_TYPE) || (filetypeid == ANALYZE75_FILE_TYPE) ||
        (filetypeid == NIFTI_IMG_TYPE)) return true;
    else return false;
}


scriptObj.printMatrixToLog = function(matrix) {

    BrainVoyagerQX.PrintToLog("Matrix:");
    for (i=0; i<4; i++) {
            BrainVoyagerQX.PrintToLog(matrix[i][0] + ", " + matrix[i][1] + ", " + matrix[i][2] + ", " + matrix[i][3]); 
    }
}

scriptObj.onlyHasNumericValues = function(matrix) {

    var value;
    var hasOnlyNumericValues = true;
    for (i=0; i<4; i++) {
        for (j=0; j<4; j++) {
           if (isNaN(matrix[i][j])) {
                hasOnlyNumericValues = false;
                break;
            }
        }            
    }
    return hasOnlyNumericValues;
}

scriptObj.setMatrixParametersForPlugin = function(matrix) {

    BrainVoyagerQX.SetPluginFloatParameter("m00", matrix[0][0]);
    BrainVoyagerQX.SetPluginFloatParameter("m01", matrix[0][1]);
    BrainVoyagerQX.SetPluginFloatParameter("m02", matrix[0][2]);
    BrainVoyagerQX.SetPluginFloatParameter("m03", matrix[0][3]);

    BrainVoyagerQX.SetPluginFloatParameter("m10", matrix[1][0]);
    BrainVoyagerQX.SetPluginFloatParameter("m11", matrix[1][1]);
    BrainVoyagerQX.SetPluginFloatParameter("m12", matrix[1][2]);
    BrainVoyagerQX.SetPluginFloatParameter("m13", matrix[1][3]);

    BrainVoyagerQX.SetPluginFloatParameter("m20", matrix[2][0]);
    BrainVoyagerQX.SetPluginFloatParameter("m21", matrix[2][1]);
    BrainVoyagerQX.SetPluginFloatParameter("m22", matrix[2][2]);
    BrainVoyagerQX.SetPluginFloatParameter("m23", matrix[2][3]);

    BrainVoyagerQX.SetPluginFloatParameter("m30", matrix[3][0]);
    BrainVoyagerQX.SetPluginFloatParameter("m31", matrix[3][1]);
    BrainVoyagerQX.SetPluginFloatParameter("m32", matrix[3][2]);
    BrainVoyagerQX.SetPluginFloatParameter("m33", matrix[3][3]);
}


scriptObj.getMatrixParametersFromPlugin = function() {

    var matrix = [[0,0,0,0],[0,0,0,0],[0,0,0,0],[0,0,0,0]]; 
    matrix[0][0] = BrainVoyagerQX.GetPluginFloatParameter("m00");
    matrix[0][1] = BrainVoyagerQX.GetPluginFloatParameter("m01");
    matrix[0][2] = BrainVoyagerQX.GetPluginFloatParameter("m02");
    matrix[0][3] = BrainVoyagerQX.GetPluginFloatParameter("m03");
             
    matrix[1][0] = BrainVoyagerQX.GetPluginFloatParameter("m10");
    matrix[1][1] = BrainVoyagerQX.GetPluginFloatParameter("m11");
    matrix[1][2] = BrainVoyagerQX.GetPluginFloatParameter("m12");
    matrix[1][3] = BrainVoyagerQX.GetPluginFloatParameter("m13");
    
    matrix[2][0] = BrainVoyagerQX.GetPluginFloatParameter("m20");
    matrix[2][1] = BrainVoyagerQX.GetPluginFloatParameter("m21");
    matrix[2][2] = BrainVoyagerQX.GetPluginFloatParameter("m22");
    matrix[2][3] = BrainVoyagerQX.GetPluginFloatParameter("m23");
    
    matrix[3][0] = BrainVoyagerQX.GetPluginFloatParameter("m30");
    matrix[3][1] = BrainVoyagerQX.GetPluginFloatParameter("m31");
    matrix[3][2] = BrainVoyagerQX.GetPluginFloatParameter("m32");
    matrix[3][3] = BrainVoyagerQX.GetPluginFloatParameter("m33");        
    
    return matrix;
}

scriptObj.getIdentityMatrix = function() {

    var matrix = [[1.0,0,0,0],[0,1.0,0,0],[0,0,1.0,0],[0,0,0,1.0]]; 
    return matrix; 
}

scriptObj.getEmptyMatrix = function() {

    var matrix = [[0,0,0,0],[0,0,0,0],[0,0,0,0],[0,0,0,0]]; 
    return matrix; 
}

scriptObj.multiply4x4Matrices = function(mat1, mat2) {

        var mat3 = this.getEmptyMatrix();
		mat3[0][0] = mat1[0][0]*mat2[0][0]+ mat1[0][1]*mat2[1][0]+ mat1[0][2]*mat2[2][0]+ mat1[0][3]*mat2[3][0];
		mat3[0][1] = mat1[0][0]*mat2[0][1]+ mat1[0][1]*mat2[1][1]+ mat1[0][2]*mat2[2][1]+ mat1[0][3]*mat2[3][1];
		mat3[0][2] = mat1[0][0]*mat2[0][2]+ mat1[0][1]*mat2[1][2]+ mat1[0][2]*mat2[2][2]+ mat1[0][3]*mat2[3][2];
		mat3[0][3] = mat1[0][0]*mat2[0][3]+ mat1[0][1]*mat2[1][3]+ mat1[0][2]*mat2[2][3]+ mat1[0][3]*mat2[3][3];
        
		mat3[1][0] = mat1[1][0]*mat2[0][0]+ mat1[1][1]*mat2[1][0]+ mat1[1][2]*mat2[2][0]+ mat1[1][3]*mat2[3][0];
		mat3[1][1] = mat1[1][0]*mat2[0][1]+ mat1[1][1]*mat2[1][1]+ mat1[1][2]*mat2[2][1]+ mat1[1][3]*mat2[3][1];
		mat3[1][2] = mat1[1][0]*mat2[0][2]+ mat1[1][1]*mat2[1][2]+ mat1[1][2]*mat2[2][2]+ mat1[1][3]*mat2[3][2];
		mat3[1][3] = mat1[1][0]*mat2[0][3]+ mat1[1][1]*mat2[1][3]+ mat1[1][2]*mat2[2][3]+ mat1[1][3]*mat2[3][3];
		
        mat3[2][0] = mat1[2][0]*mat2[0][0]+ mat1[2][1]*mat2[1][0]+ mat1[2][2]*mat2[2][0]+ mat1[2][3]*mat2[3][0];
		mat3[2][1] = mat1[2][0]*mat2[0][1]+ mat1[2][1]*mat2[1][1]+ mat1[2][2]*mat2[2][1]+ mat1[2][3]*mat2[3][1];
		mat3[2][2] = mat1[2][0]*mat2[0][2]+ mat1[2][1]*mat2[1][2]+ mat1[2][2]*mat2[2][2]+ mat1[2][3]*mat2[3][2];
		mat3[2][3] = mat1[2][0]*mat2[0][3]+ mat1[2][1]*mat2[1][3]+ mat1[2][2]*mat2[2][3]+ mat1[2][3]*mat2[3][3];
		
        mat3[3][0] = mat1[3][0]*mat2[0][0]+ mat1[3][1]*mat2[1][0]+ mat1[3][2]*mat2[2][0]+ mat1[3][3]*mat2[3][0];
		mat3[3][1] = mat1[3][0]*mat2[0][1]+ mat1[3][1]*mat2[1][1]+ mat1[3][2]*mat2[2][1]+ mat1[3][3]*mat2[3][1];
		mat3[3][2] = mat1[3][0]*mat2[0][2]+ mat1[3][1]*mat2[1][2]+ mat1[3][2]*mat2[2][2]+ mat1[3][3]*mat2[3][2];
		mat3[3][3] = mat1[3][0]*mat2[0][3]+ mat1[3][1]*mat2[1][3]+ mat1[3][2]*mat2[2][3]+ mat1[3][3]*mat2[3][3];
        return mat3;
}

function getdeterminant(a){
// Get determinant of 4 x 4 matrix
// courtesy of John A.
  var dt = mat[0][0]*( mat[1][1]*( mat[2][2]*mat[3][3]-mat[2][3]*mat[3][2])
		+mat[2][1]*(-mat[1][2]*mat[3][3]+mat[1][3]*mat[3][2])
		+mat[3][1]*( mat[1][2]*mat[2][3]-mat[1][3]*mat[2][2]))
        +mat[1][0]*( mat[0][1]*(-mat[2][2]*mat[3][3]+mat[2][3]*mat[3][2])
		+mat[2][1]*( mat[0][2]*mat[3][3]-mat[0][3]*mat[3][2])
		+mat[3][1]*(-mat[0][2]*mat[2][3]+mat[0][3]*mat[2][2]))
        +mat[2][0]*( mat[0][1]*( mat[1][2]*mat[3][3]-mat[1][3]*mat[3][2])
		+mat[1][1]*(-mat[0][2]*mat[3][3]+mat[0][3]*mat[3][2])
		+mat[3][1]*( mat[0][2]*mat[1][3]-mat[0][3]*mat[1][2]))
        +mat[3][0]*( mat[0][1]*(-mat[1][2]*mat[2][3]+mat[1][3]*mat[2][2])
		+mat[1][1]*( mat[0][2]*mat[2][3]-mat[0][3]*mat[2][2])
		+mat[2][1]*(-mat[0][2]*mat[1][3]+mat[0][3]*mat[1][2]));
		return(dt);
}

function getinverse(mat){
// get inverse of 4 x 4 matrix
// courtesy of John A.

	var idet    = 1.0/getdeterminant(mat);
    var invmat = getIdentityMatrix();

    invmat[0][0] = idet*(mat[1][1]*( mat[2][2]*mat[3][3]-mat[2][3]*mat[3][2])
		+mat[2][1]*(-mat[1][2]*mat[3][3]+mat[1][3]*mat[3][2])
		+mat[3][1]*( mat[1][2]*mat[2][3]-mat[1][3]*mat[2][2]));
    invmat[0][1] = idet*(mat[0][1]*(-mat[2][2]*mat[3][3]+mat[2][3]*mat[3][2])
		+mat[2][1]*( mat[0][2]*mat[3][3]-mat[0][3]*mat[3][2])
		+mat[3][1]*(-mat[0][2]*mat[2][3]+mat[0][3]*mat[2][2]));
    invmat[0][2] = idet*(mat[0][1]*( mat[1][2]*mat[3][3]-mat[1][3]*mat[3][2])
		+mat[1][1]*(-mat[0][2]*mat[3][3]+mat[0][3]*mat[3][2])
		+mat[3][1]*( mat[0][2]*mat[1][3]-mat[0][3]*mat[1][2]));
    invmat[0][3] = idet*(mat[0][1]*(-mat[1][2]*mat[2][3]+mat[1][3]*mat[2][2])
		+mat[1][1]*( mat[0][2]*mat[2][3]-mat[0][3]*mat[2][2])
		+mat[2][1]*(-mat[0][2]*mat[1][3]+mat[0][3]*mat[1][2]));
    invmat[1][0] = idet*(mat[1][0]*(-mat[2][2]*mat[3][3]+mat[2][3]*mat[3][2])
		+mat[2][0]*( mat[1][2]*mat[3][3]-mat[1][3]*mat[3][2])
		+mat[3][0]*(-mat[1][2]*mat[2][3]+mat[1][3]*mat[2][2]));
    invmat[1][1] = idet*(mat[0][0]*( mat[2][2]*mat[3][3]-mat[2][3]*mat[3][2])
		+mat[2][0]*(-mat[0][2]*mat[3][3]+mat[0][3]*mat[3][2])
		+mat[3][0]*( mat[0][2]*mat[2][3]-mat[0][3]*mat[2][2]));
    invmat[1][2] = idet*(mat[0][0]*(-mat[1][2]*mat[3][3]+mat[1][3]*mat[3][2])
		+mat[1][0]*( mat[0][2]*mat[3][3]-mat[0][3]*mat[3][2])
		+mat[3][0]*(-mat[0][2]*mat[1][3]+mat[0][3]*mat[1][2]));
    invmat[1][3] = idet*(mat[0][0]*( mat[1][2]*mat[2][3]-mat[1][3]*mat[2][2])
		+mat[1][0]*(-mat[0][2]*mat[2][3]+mat[0][3]*mat[2][2])
		+mat[2][0]*( mat[0][2]*mat[1][3]-mat[0][3]*mat[1][2]));
    invmat[2][0] = idet*(mat[1][0]*( mat[2][1]*mat[3][3]-mat[2][3]*mat[3][1])
		+mat[2][0]*(-mat[1][1]*mat[3][3]+mat[1][3]*mat[3][1])
		+mat[3][0]*( mat[1][1]*mat[2][3]-mat[1][3]*mat[2][1]));
    invmat[2][1] = idet*(mat[0][0]*(-mat[2][1]*mat[3][3]+mat[2][3]*mat[3][1])
		+mat[2][0]*( mat[0][1]*mat[3][3]-mat[0][3]*mat[3][1])
		+mat[3][0]*(-mat[0][1]*mat[2][3]+mat[0][3]*mat[2][1]));
    invmat[2][2] = idet*(mat[0][0]*( mat[1][1]*mat[3][3]-mat[1][3]*mat[3][1])
		+mat[1][0]*(-mat[0][1]*mat[3][3]+mat[0][3]*mat[3][1])
		+mat[3][0]*( mat[0][1]*mat[1][3]-mat[0][3]*mat[1][1]));
    invmat[2][3] = idet*(mat[0][0]*(-mat[1][1]*mat[2][3]+mat[1][3]*mat[2][1])
		+mat[1][0]*( mat[0][1]*mat[2][3]-mat[0][3]*mat[2][1])
		+mat[2][0]*(-mat[0][1]*mat[1][3]+mat[0][3]*mat[1][1]));
    invmat[3][0] = idet*(mat[1][0]*(-mat[2][1]*mat[3][2]+mat[2][2]*mat[3][1])
		+mat[2][0]*( mat[1][1]*mat[3][2]-mat[1][2]*mat[3][1])
		+mat[3][0]*(-mat[1][1]*mat[2][2]+mat[1][2]*mat[2][1]));
    invmat[3][1] = idet*(mat[0][0]*( mat[2][1]*mat[3][2]-mat[2][2]*mat[3][1])
		+mat[2][0]*(-mat[0][1]*mat[3][2]+mat[0][2]*mat[3][1])
		+mat[3][0]*( mat[0][1]*mat[2][2]-mat[0][2]*mat[2][1]));
    invmat[3][2] = idet*(mat[0][0]*(-mat[1][1]*mat[3][2]+mat[1][2]*mat[3][1])
		+mat[1][0]*( mat[0][1]*mat[3][2]-mat[0][2]*mat[3][1])
		+mat[3][0]*(-mat[0][1]*mat[1][2]+mat[0][2]*mat[1][1]));
    invmat[3][3] = idet*(mat[0][0]*( mat[1][1]*mat[2][2]-mat[1][2]*mat[2][1])
		+mat[1][0]*(-mat[0][1]*mat[2][2]+mat[0][2]*mat[2][1])
		+mat[2][0]*( mat[0][1]*mat[1][2]-mat[0][2]*mat[1][1]));
        
    return invmat;
}

scriptObj.getPath = function(filename) {
    
    var start = 0; // start searching from begin 
    var last = filename.lastIndexOf("/"); // search for last path separator 
    var path = filename.substring(start, (last+1)); // now return path without filename 
    return path; 
}

scriptObj.removeExtension = function(filename) {

    var start = 0; // start searching from begin 
    var posExt = filename.lastIndexOf("."); // search for `extension separator' 
    var nameWithoutExt = filename.substring(start, posExt); // get name without extension 
    return nameWithoutExt;
}

scriptObj.getExtension = function(filename) {

    var start = 0; // start searching from begin 
    var posExt = filename.lastIndexOf("."); // search for `extension separator' 
    var ext = filename.substring(posExt, filename.length); // get extension without name, including '.'
    return ext;
}

// exchange extension of filename; new extension should contain '.', like ".txt"
scriptObj.exchangeExtension = function(filename, newExt) {

    var newName = this.removeExtension(filename) + newExt;
    return newName;
}

// add a addition to filename: <filename>_addition.<ext>    
scriptObj.changeFileName = function(filename, addition) {

    var nameWithoutExt = this.removeExtension(filename);
    var newName = nameWithoutExt + "_" + addition + this.getExtension(filename);
    return newName;
}    

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 1. CONVERSIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  

scriptObj.prepareConversion = function() {

    BrainVoyagerQX.PrintToLog("Prepare conversion...");
    
    // get information from widgets
    var fromID = this.getConvertFromFileId(); 
    var toID = this.getConvertToFileId(); 
    var doTransform = transformBox.checked;
    var doFlip = flipBox.checked;    
    doBatch = batchBox.checked;
    openInBVQX = openinbvqxBox.checked;
    var doZeropad = zeropadBox.checked;
    var adaptIntensity = intensityBox.checked;
    var adaptInterpolation = vointerpBox.checked;
    var resetOrient = resetBox.checked;
    var selectVols = chbSelectVols.checked;
        
    // set parameters for plugin // const CONVERT_REGISTERED = 3; // const CONVERT_REGISTERED_BATCH = 4;
    if ((fromID != FILE_TYPE_UNKNOWN) && (toID != FILE_TYPE_UNKNOWN)) { 
        BrainVoyagerQX.SetPluginStringParameter("MenuOption", "Convert"); 
        BrainVoyagerQX.SetPluginIntParameter("SubmenuOption", (doBatch ? CONVERT_BATCH : CONVERT_SINGLE_FILE));
        BrainVoyagerQX.SetPluginIntParameter("ConvertFromID", fromID);
        BrainVoyagerQX.SetPluginIntParameter("ConvertToID", toID);
        BrainVoyagerQX.SetPluginIntParameter("DoFlip", doFlip);
        BrainVoyagerQX.SetPluginIntParameter("DoTransform", doTransform);
        BrainVoyagerQX.SetPluginIntParameter("DoZeropad", doZeropad);
        BrainVoyagerQX.SetPluginIntParameter("OpenInBVQX", openInBVQX);
        BrainVoyagerQX.SetPluginIntParameter("AdaptIntensity", adaptIntensity);
        BrainVoyagerQX.SetPluginIntParameter("AdaptInterpolation", adaptInterpolation);
        BrainVoyagerQX.SetPluginIntParameter("ResetOrientation", resetOrient);
        BrainVoyagerQX.SetPluginIntParameter("SelectVolsTimeSeries", selectVols);
        
        if (selectVols) {
            var fromVol = spbFromVol.value;
            var toVol = spbToVol.value;
            if (toVol < fromVol) {
                BrainVoyagerQX.PrintToLog("Cannot select negative number of volumes in timeseries. Values ignored.");
                BrainVoyagerQX.SetPluginIntParameter("SelectVolsTimeSeries", 0);
            } else {
                BrainVoyagerQX.SetPluginIntParameter("FromVolInTimeSeries", fromVol);
                BrainVoyagerQX.SetPluginIntParameter("ToVolInTimeSeries", toVol);
            }
        }
        
        if (this.isniftifile(fromID)) {
            if (hasniftifile) {
                BrainVoyagerQX.SetPluginStringParameter("NiftiFileName", niftiNameLabel.text); 
            } else {
                BrainVoyagerQX.PrintToLog("Could not find NIfTI file name to convert.");
                incompleteinfo = true;
            }
        }
    }
}

scriptObj.getConvertFromFileId = function() {
    
    var fileid;
    if (niiFromButton.checked == 1) {
        fileid = NIFTI_FILE_TYPE;
    } else if (hdrFromButton.checked == 1) {
       fileid = NIFTI_PAIR_FILE_TYPE;
    } else if (vmrFromButton.checked == 1) {
       fileid = BV_TYPE_VMR;
    } else if (fmrFromButton.checked == 1) {
       fileid = BV_TYPE_FMR;
    } else if (dmrFromButton.checked == 1) {
       fileid = BV_TYPE_DMR;
    } else if (vtcFromButton.checked == 1) {
       fileid = BV_TYPE_VTC;
    } else if (vmpFromButton.checked == 1) {
       fileid = BV_TYPE_VMP;
    } else if (voiFromButton.checked == 1) {
       fileid = BV_TYPE_VOI;
    } else {
        fileid = FILE_TYPE_UNKNOWN;
    }
    return fileid;
}

scriptObj.getConvertToFileId = function() {
  
    var fileid;
    if (niiToButton.checked == 1) {
        fileid = NIFTI_FILE_TYPE;
    } else if (hdrToButton.checked == 1) {
       fileid = NIFTI_PAIR_FILE_TYPE;
    } else if (vmrToButton.checked == 1) {
       fileid = BV_TYPE_VMR;
    } else if (fmrToButton.checked == 1) {
       fileid = BV_TYPE_FMR;
    } else if (dmrToButton.checked == 1) {
       fileid = BV_TYPE_DMR;
    } else if (vtcToButton.checked == 1) {
       fileid = BV_TYPE_VTC;
    } else if (vmpToButton.checked == 1) {
       fileid = BV_TYPE_VMP;
    } else {
        fileid = FILE_TYPE_UNKNOWN;
    }
    return fileid;
}

scriptObj.getBatchFile = function() {

    BrainVoyagerQX.SetPluginStringParameter("Command", "GetFile");
    BrainVoyagerQX.SetPluginIntParameter("FileID", FILE_TYPE_TXT);
    BrainVoyagerQX.ExecutePlugin(); 
    var batchfilename = BrainVoyagerQX.GetPluginStringParameter("BatchFileName");
    batchFileLabel.text = batchfilename;
}


scriptObj.enableBatchButton = function() {

    var doBatch = batchBox.checked;
    if (doBatch) getBatchConvButton.enabled = true;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 2. INFORMATION FUNCTIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  

scriptObj.prepareCollectInfo = function() {

    // get information from widgets
    var success = true;
    var submenuoption;
    var fileid = this.getInfoFileId();
    if (destLogButton.checked == 1) {
        submenuoption = PRINT_TO_LOG;
    } else if (destFileButton.checked == 1) {
        submenuoption = PRINT_TO_FILE;
    } else if (destBothButton.checked == 1) {
        submenuoption = PRINT_TO_LOG_AND_FILE;
    } else {
        incompleteinfo = true;
        success = false;
    }
  
    // set parameters for plugin 
    if ((fileid != FILE_TYPE_UNKNOWN) && (!incompleteinfo)) {
    
        BrainVoyagerQX.SetPluginStringParameter("MenuOption", "PrintInfo"); 
        BrainVoyagerQX.SetPluginIntParameter("FileID", fileid);
        BrainVoyagerQX.SetPluginIntParameter("SubmenuOption", submenuoption);
        if (this.isniftifile(fileid)) {
            if (hasniftifile) {        
                BrainVoyagerQX.SetPluginStringParameter("NiftiFileName", niftiNameLabel.text); 
            } else {
                incompleteinfo = true;
                success = false;
            }
        }
    } else {
        success = false;
    }
    return success;
}

scriptObj.getInfoFileId = function() {

    var fileid;
    if (niiPrintButton.checked == 1) {
        fileid = NIFTI_FILE_TYPE;
    } else if (hdrPrintButton.checked == 1) {
       fileid = NIFTI_PAIR_FILE_TYPE;
    } else if (vmrPrintButton.checked == 1) {
       fileid = BV_TYPE_VMR;
    } else if (fmrPrintButton.checked == 1) {
       fileid = BV_TYPE_FMR;
    } else if (dmrPrintButton.checked == 1) {
       fileid = BV_TYPE_DMR;
    } else if (vtcPrintButton.checked == 1) {
       fileid = BV_TYPE_VTC;
    } else if (vmpPrintButton.checked == 1) {
       fileid = BV_TYPE_VMP;
    } else if (voiPrintButton.checked == 1) {
       fileid = BV_TYPE_VOI;
    } else {
        fileid = FILE_TYPE_UNKNOWN;
    }
    return fileid;
}


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 3. POSITIONING FUNCTIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
 
scriptObj.getPosFile = function() {
 
    BrainVoyagerQX.SetPluginStringParameter("Command", "GetFile");
    BrainVoyagerQX.SetPluginIntParameter("FileID", BV_TYPE_POS);
    BrainVoyagerQX.ExecutePlugin(); 
    var posfilename = BrainVoyagerQX.GetPluginStringParameter("PosFileName");
    posFileLabel.text = posfilename;
    hasposfile = true;
}
          
scriptObj.preparePosFuncs = function() {

    // get information from widgets
    var submenuoption, matrix;
    if (printPosMatButton.checked == 1) {
        submenuoption = POS_MATRIX_FROM_FILE;
        if (hasposfile) {
            BrainVoyagerQX.SetPluginStringParameter("PosFileName", posFileLabel.text);
        } else {
            incompleteinfo = true;
        }
    } else if (createPosFileButton.checked == 1) {
        submenuoption = POS_ADD_TO_FILE;
        matrix = this.getPosMatrix();
        this.printMatrixToLog(matrix);
        if (!this.onlyHasNumericValues(matrix)) {
            BrainVoyagerQX.PrintToLog("Cannot proceed: values of positioning matrix are not numeric.");
            incompleteinfo = true;
        } else {
            BrainVoyagerQX.SetPluginIntParameter("nx", nxBox.value);
            BrainVoyagerQX.SetPluginIntParameter("ny", nyBox.value);
            BrainVoyagerQX.SetPluginIntParameter("nz", nzBox.value);                        
            BrainVoyagerQX.SetPluginFloatParameter("dx", dxBox.value);
            BrainVoyagerQX.SetPluginFloatParameter("dy", dyBox.value);
            BrainVoyagerQX.SetPluginFloatParameter("dz", dzBox.value);                        
        }       
    } else if ((submenuoption == POS_MATRIX_FROM_FILE) && (!hasposfile)) {
        BrainVoyagerQX.PrintToLog("Cannot proceed: no positioning file selected."); 
    } else {
        incompleteinfo = true;
    }
    
    // set parameters for plugin
    if (!incompleteinfo) {
        BrainVoyagerQX.SetPluginStringParameter("MenuOption", "PosFuncs"); 
        BrainVoyagerQX.SetPluginIntParameter("SubmenuOption", submenuoption);
        if (submenuoption == POS_ADD_TO_FILE) this.setMatrixParametersForPlugin(matrix);
    }
}

scriptObj.getPosMatrix = function() {

	BrainVoyagerQX.PrintToLog("Get positioning matrix...");
    var matrix = [[0,0,0,0],[0,0,0,0],[0,0,0,0],[0,0,0,0]]; 
    matrix[0][0] = p00.text;
    matrix[0][1] = p01.text;
    matrix[0][2] = p02.text;
    matrix[0][3] = p03.text;
    matrix[1][0] = p10.text;
    matrix[1][1] = p11.text;
    matrix[1][2] = p12.text;
    matrix[1][3] = p13.text;
    matrix[2][0] = p20.text;
    matrix[2][1] = p21.text;
    matrix[2][2] = p22.text;
    matrix[2][3] = p23.text;
    matrix[3][0] = p30.text;
    matrix[3][1] = p31.text;
    matrix[3][2] = p32.text;
    matrix[3][3] = p33.text;    
    return matrix;
}

scriptObj.writePosFile = function(imgFileName, firstSlicePosMat, lastSlicePosMat) {

    var posfilename = this.changeFileName(this.exchangeExtension(imgFileName, ".txt"), "posinfo_qtscript"); 
    var posfile = new QFile(posfilename);
    posfile.open(new QIODevice.OpenMode(QIODevice.WriteOnly)); 
    var fid = new QTextStream(posfile);
    fid.writeString("\nPositioning information from NIfTI file:\n\n");
    fid.writeString("Slice1CenterX:\t\t"); fid.writeDouble(firstSlicePosMat[0][3]); fid.writeString("\t\t\n");
    fid.writeString("Slice1CenterY:\t\t"); fid.writeDouble(firstSlicePosMat[1][3]); fid.writeString("\t\t\n");
    fid.writeString("Slice1CenterZ:\t\t"); fid.writeDouble(firstSlicePosMat[2][3]); fid.writeString("\t\t\n");
    fid.writeString("SliceNCenterX:\t\t"); fid.writeDouble(lastSlicePosMat[0][3]); fid.writeString("\t\t\n");
    fid.writeString("SliceNCenterY:\t\t"); fid.writeDouble(lastSlicePosMat[1][3]); fid.writeString("\t\t\n");
    fid.writeString("SliceNCenterZ:\t\t"); fid.writeDouble(lastSlicePosMat[2][3]); fid.writeString("\t\t\n");    
    fid.writeString("RowDirX:\t\t"); fid.writeDouble(firstSlicePosMat[0][0]); fid.writeString("\t\t\n");
    fid.writeString("RowDirY:\t\t"); fid.writeDouble(firstSlicePosMat[1][0]); fid.writeString("\t\t\n");
    fid.writeString("RowDirZ:\t\t"); fid.writeDouble(firstSlicePosMat[2][0]); fid.writeString("\t\t\n");
    fid.writeString("ColDirX:\t\t"); fid.writeDouble(firstSlicePosMat[0][1]); fid.writeString("\t\t\n");
    fid.writeString("ColDirY:\t\t"); fid.writeDouble(firstSlicePosMat[1][1]); fid.writeString("\t\t\n");
    fid.writeString("ColDirZ:\t\t"); fid.writeDouble(firstSlicePosMat[2][1]); fid.writeString("\t\t\n");
    posfile.close();
    BrainVoyagerQX.PrintToLog("Wrote file: " + posfilename);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 4. HEADER MANIPULATION FUNCTIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  

scriptObj.prepareHdrManipFuncs = function() {

    var niftitarget = niftiNameLabel.text; // for both submenuoptions    
    if (niftitarget.length != 0) {
           BrainVoyagerQX.SetPluginStringParameter("NiftiFileName", niftitarget); 
           BrainVoyagerQX.SetPluginIntParameter("FileID", this.getManipulatedFileType());
    } else {
            incompleteinfo = true;
    }

    var submenuoption;    
    if (optCopyButton.checked == 1) {
        
        submenuoption = 1;
        var copyall = (rbtnCopyAll.checked == 1);
        BrainVoyagerQX.SetPluginIntParameter("CopyAll", (copyall ? 1 : 0));

        var niftisource = lblOtherNIfTIfile.text;
        if (niftisource.length != 0) {
            BrainVoyagerQX.SetPluginStringParameter("NiftiSourceFileName", niftisource); 
            BrainVoyagerQX.SetPluginIntParameter("SrcFileID", this.getFiletypeToCopy());
        } else {
            incompleteinfo = true;
        }
        
    } else if (optFldManipButton.checked == 1) {
    
        submenuoption = 2;
        matrix = this.getSformMatrix();
        if (this.onlyHasNumericValues(matrix)) {
            this.setMatrixParametersForPlugin(matrix);
        } else {
            incompleteinfo = true; 
        }
        BrainVoyagerQX.SetPluginIntParameter("QFormCode", leQformCode.text);
        BrainVoyagerQX.SetPluginIntParameter("SFormCode", leSformCode.text);
        BrainVoyagerQX.SetPluginFloatParameter("Qb", leQb.text);
        BrainVoyagerQX.SetPluginFloatParameter("Qc", leQc.text);
        BrainVoyagerQX.SetPluginFloatParameter("Qd", leQd.text);
        BrainVoyagerQX.SetPluginFloatParameter("QoffsetX", leQoffsetX.text);
        BrainVoyagerQX.SetPluginFloatParameter("QoffsetY", leQoffsetY.text);
        BrainVoyagerQX.SetPluginFloatParameter("QoffsetZ", leQoffsetZ.text);
        
    }
    
    if (!incompleteinfo) {
    
        BrainVoyagerQX.PrintToLog("Set plugin parameters for header manipulation function...");
        BrainVoyagerQX.SetPluginStringParameter("MenuOption", "HdrFuncs"); 
        BrainVoyagerQX.SetPluginIntParameter("SubmenuOption", submenuoption);
    }
}

scriptObj.getOtherNiftiFile = function() {

    var filetypeid = this.getFiletypeToCopy();                        
    if (filetypeid != FILE_TYPE_UNKNOWN) {          
        BrainVoyagerQX.SetPluginStringParameter("Command", "GetFile");    
        BrainVoyagerQX.SetPluginIntParameter("FileID", filetypeid);
        BrainVoyagerQX.ExecutePlugin(); 
        var niftifilename = BrainVoyagerQX.GetPluginStringParameter("NiftiFileName");
        lblOtherNIfTIfile.text = niftifilename; 
    }    
}

scriptObj.getSformMatrix = function() {

	BrainVoyagerQX.PrintToLog("Get S-form matrix...");
    var matrix = [[0,0,0,0],[0,0,0,0],[0,0,0,0],[0,0,0,0]]; 
    matrix[0][0] = leSrow00.text;
    matrix[0][1] = leSrow01.text;
    matrix[0][2] = leSrow02.text;
    matrix[0][3] = leSrow03.text;
    matrix[1][0] = leSrow10.text;
    matrix[1][1] = leSrow11.text;
    matrix[1][2] = leSrow12.text;
    matrix[1][3] = leSrow13.text;
    matrix[2][0] = leSrow20.text;
    matrix[2][1] = leSrow21.text;
    matrix[2][2] = leSrow22.text;
    matrix[2][3] = leSrow23.text;
    matrix[3][0] = leSrow30.text;
    matrix[3][1] = leSrow31.text;
    matrix[3][2] = leSrow32.text;
    matrix[3][3] = leSrow33.text;    
    return matrix;
}

scriptObj.setSformMatrix = function(matrix) {

    leSrow00.text = matrix[0][0];
    leSrow01.text = matrix[0][1];
    leSrow02.text = matrix[0][2];
    leSrow03.text = matrix[0][3];    

    leSrow10.text = matrix[1][0];
    leSrow11.text = matrix[1][1];
    leSrow12.text = matrix[1][2];
    leSrow13.text = matrix[1][3]; 
    
    leSrow20.text = matrix[2][0];
    leSrow21.text = matrix[2][1];
    leSrow22.text = matrix[2][2];
    leSrow23.text = matrix[2][3]; 
    
    leSrow30.text = matrix[3][0];
    leSrow31.text = matrix[3][1];
    leSrow32.text = matrix[3][2];
    leSrow33.text = matrix[3][3];             
}

scriptObj.getManipulatedFileType = function() {

    var filetypeid;
    if (rbtnNiiTarget.checked == 1) {
        filetypeid = NIFTI_FILE_TYPE;
    } else if (rbtnHdrTarget.checked == 1) {
        filetypeid = NIFTI_PAIR_FILE_TYPE;
    }
    return filetypeid;
} 

scriptObj.getFiletypeToCopy = function() {
  
    var filetypeid;
    if (rbtnNiiType.checked == 1) {
        filetypeid = NIFTI_FILE_TYPE;
    } else if (rbtnHdrType.checked == 1) {
        filetypeid = NIFTI_PAIR_FILE_TYPE;
    }
    return filetypeid;
}

 // this function should be activated by a button click on the NIfTI Header manip tab
scriptObj.calculateBrainVoyagerPositionFromNIfTI = function() { 

    if (hasniftifile) {

        BrainVoyagerQX.PrintToLog("Start calculating position in BrainVoyager coordinates...");
        var niipos = this.getSformMatrix();
this.printMatrixToLog(niipos);        

        // transfer info to local variables
        var voxx = lblDx.text;
        var voxy = lblDy.text;
        var voxz = lblDz.text;
        var dimx = lblNx.text; 
        var dimy = lblNy.text;
        var dimz = lblNz.text;

        // create nifti positioning matrix
     /*   var axial = false; var sagit = false; var coronal = false;
        var orient1 = Math.abs(Math.round(niipos[0][0]));
        var orient2 = Math.abs(Math.round(niipos[1][0]));
        var orient3 = Math.abs(Math.round(niipos[2][0]));
        if ((orient1 > orient2) && (orient1 > orient3)) {
            axial = true;
            BrainVoyagerQX.PrintToLog("Assume axial orientation w.r.t. NIfTI coordinate system");
        } else if ((orient2 > orient1) && (orient2 > orient3)) {
            sagit = true;
            BrainVoyagerQX.PrintToLog("Assume sagittal orientation w.r.t. NIfTI coordinate system");
        } else {
            coronal = true;
            BrainVoyagerQX.PrintToLog("Assume coronal orientation w.r.t. NIfTI coordinate system");
        }
*/
        var nii2dcm = this.getIdentityMatrix(); 
    /*    if ((axial) || (coronal)) { // no solution for coronal
            nii2dcm[0][0] = -1;
            nii2dcm[0][3] = (dimx - 1);
        } else if (sagit) {
            nii2dcm[0][0] = -1;
            nii2dcm[0][3] = (dimx - 1);
            nii2dcm[1][1] = -1;
            nii2dcm[1][3] = (dimy - 1);
        }    */
                 nii2dcm[0][0] = -1;
            nii2dcm[0][3] = dimx;
            nii2dcm[1][1] = -1;
            nii2dcm[1][3] = (dimy - 1);
     //    nii2dcm = [-1 0 0 dim(2);
       //     0 -1 0 dim(3)-1;
         //   0 0 1 0;  
           // 0 0 0 1]
this.printMatrixToLog(nii2dcm);

        // from nifti2dicom
        var dcmposmatrix = this.multiply4x4Matrices(niipos, nii2dcm);
this.printMatrixToLog(dcmposmatrix);        


        // from dicom2brainvoyager
        var dcm2bv1 = this.getIdentityMatrix();
        dcm2bv1[0][0] = 1.0/voxx;
        dcm2bv1[1][1] = 1.0/voxy;
        dcm2bv1[2][2] = 1.0/voxz;
        dcm2bv1[0][3] = (dimx/2.0);
        dcm2bv1[1][3] = (dimy/2.0);        
        var first = this.multiply4x4Matrices(dcmposmatrix, dcm2bv1); 
this.printMatrixToLog(first);        

        var dcm2bvn = this.getIdentityMatrix();
        dcm2bvn[0][0] = 1.0/voxx;
        dcm2bvn[1][1] = 1.0/voxy;
        dcm2bvn[2][2] = 1.0/voxz;
        dcm2bvn[0][3] = (dimx/2.0);
        dcm2bvn[1][3] = (dimy/2.0);
        dcm2bvn[2][3] = (dimz - 1.0);
        var last = this.multiply4x4Matrices(dcmposmatrix, dcm2bvn);  
this.printMatrixToLog(last);        

        // write to disk
        var niftifilename = niftiNameLabel.text;
        var success = this.writePosFile(niftifilename, first, last);
        BrainVoyagerQX.PrintToLog("Finished calculating positioning information.\nText file available in same directory as image file.");            
    } else {
        BrainVoyagerQX.PrintToLog("No NIfTI file available. Position calculation cancelled.");
    }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 5. TRANSFORMATION FUNCTIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 

scriptObj.getTrfFile = function() {

    BrainVoyagerQX.SetPluginStringParameter("Command", "GetFile");    
    BrainVoyagerQX.SetPluginIntParameter("FileID", BV_TYPE_TRF);
    BrainVoyagerQX.ExecutePlugin(); 
    var trffilename = BrainVoyagerQX.GetPluginStringParameter("TrfFileName");
    trfFileLabel.text = trffilename;
    hastrffile = true;
}

scriptObj.prepareTrfFuncs = function() {

    // get information from widgets
    var submenuoption, matrix, fileid;
    if (invertTrfButton.checked == 1) {
        submenuoption = TRF_INVERT_MAT;
        matrix = this.getTrfMatrix();
        this.printMatrixToLog(matrix);
        if (!this.onlyHasNumericValues(matrix)) {
            BrainVoyagerQX.PrintToLog("Cannot proceed: values of transformation matrix are not numeric.");
            incompleteinfo = true;
        }        
    } else if (applyManTrfButton.checked == 1) {
        submenuoption = TRF_APPLY_TRF_MAN;
        matrix = this.getTrfMatrix();
        this.printMatrixToLog(matrix);
        if (!this.onlyHasNumericValues(matrix)) {
            BrainVoyagerQX.PrintToLog("Cannot proceed: values of positioning matrix are not numeric.");
            incompleteinfo = true;
        }        
        fileid = this.getFiletypeToTransform();
    } else if (applyBVQXTrfButton.checked == 1) {
        submenuoption = TRF_APPLY_TRF_FILE;
        fileid = this.getFiletypeToTransform();
        if (hastrffile) {
            BrainVoyagerQX.SetPluginStringParameter("TrfFileName", trfFileLabel.text);
        } else {
            incompleteinfo = true;
            BrainVoyagerQX.PrintToLog("Cannot proceed: no transformation file selected."); 
        }   
    }

    // set parameters for plugin
    if (!incompleteinfo) {
    
        BrainVoyagerQX.PrintToLog("Set plugin parameters for transformation function...");
        BrainVoyagerQX.SetPluginStringParameter("MenuOption", "TrfFuncs"); 
        BrainVoyagerQX.SetPluginIntParameter("SubmenuOption", submenuoption);
        // use manual matrix
        if (submenuoption == 1 || submenuoption == 2) {
            BrainVoyagerQX.PrintToLog("Set matrix fields...");
            this.setMatrixParametersForPlugin(matrix);       
        }
        if (submenuoption == 2 || submenuoption == 3) {        
            BrainVoyagerQX.SetPluginIntParameter("FileID", fileid);
            if (this.isniftifile(fileid)) {
                if (hasniftifile) {
                    BrainVoyagerQX.SetPluginStringParameter("NiftiFileName", niftiNameLabel.text); 
                } else {
                    incompleteinfo = true;
                }
            }                        
        }        
    } else {
        BrainVoyagerQX.PrintToLog("Information not complete.");
    }
}

scriptObj.getTrfMatrix = function() {

	BrainVoyagerQX.PrintToLog("Get transformation matrix...");
    var matrix = [[0,0,0,0],[0,0,0,0],[0,0,0,0],[0,0,0,0]]; 
    matrix[0][0] = t00.text;
    matrix[0][1] = t01.text;
    matrix[0][2] = t02.text;
    matrix[0][3] = t03.text;
    matrix[1][0] = t10.text;
    matrix[1][1] = t11.text;
    matrix[1][2] = t12.text;
    matrix[1][3] = t13.text;
    matrix[2][0] = t20.text;
    matrix[2][1] = t21.text;
    matrix[2][2] = t22.text;
    matrix[2][3] = t23.text;
    matrix[3][0] = t30.text;
    matrix[3][1] = t31.text;
    matrix[3][2] = t32.text;
    matrix[3][3] = t33.text;    
    return matrix;
}


scriptObj.getFiletypeToTransform = function() {
  
    var filetypeid;
    var currentindex = getTrfFileTypeBox.currentIndex;
    switch (currentindex) {
        case 0: filetypeid = FILE_TYPE_UNKNOWN; break;
        case 1: filetypeid = BV_TYPE_VMR; break;
        case 2: filetypeid = BV_TYPE_FMR; break;
        case 3: filetypeid = BV_TYPE_DMR; break;
        case 4: filetypeid = BV_TYPE_VTC; break;
        case 5: filetypeid = BV_TYPE_VMP; break;
        case 6: filetypeid = NIFTI_FILE_TYPE; break;
        case 7: filetypeid = NIFTI_PAIR_FILE_TYPE; break;
        default: filetypeid = FILE_TYPE_UNKNOWN;
    }
    return filetypeid;
}

scriptObj.checkTrfFileTypeBox = function() {

      var currentindex = getTrfFileTypeBox.currentIndex;
      if ((currentindex == 6)||(currentindex == 7)) {
            getNiftiFileButton.enabled = true;
      } else {
            getNiftiFileButton.enabled = false;
      }
    
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 6. BATCH FUNCTIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  

scriptObj.prepareBatchFuncs = function() {

    // set parameters for plugin
   BrainVoyagerQX.SetPluginStringParameter("MenuOption", "BatchFuncs"); 
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ MAIN ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

scriptObj.initDlg = function() { 

    this.niftiDlg.windowTitle = "NIfTI converter"; 

    // ~~~ make nicknames for children ~~~
	var bvqx = BrainVoyagerQX;
    startButton = bvqx.FindChild("btnStart"); 
    // global tab
    optConvertButton    = bvqx.FindChild("rbtnConvert");
    optInfoButton       = bvqx.FindChild("rbtnImgInfo");
    optPosButton        = bvqx.FindChild("rbtnPosFuncs");
    optManipButton      = bvqx.FindChild("rbtnHdrManip");
    optTrfButton        = bvqx.FindChild("rbtnTrfFuncs");
    optBatchButton      = bvqx.FindChild("rbtnBatchFunctions");
    getNiftiFileButton  = bvqx.FindChild("btnGetNIfTIFile");
    niftiNameLabel      = bvqx.FindChild("lblNIfTIname");
    
    // conversion tab
    niiFromButton = bvqx.FindChild("rbtnFromNII");  
    hdrFromButton = bvqx.FindChild("rbtnFromHDR");
    vmrFromButton = bvqx.FindChild("rbtnFromVMR"); 
    fmrFromButton = bvqx.FindChild("rbtnFromFMR"); 
    dmrFromButton = bvqx.FindChild("rbtnFromDMR"); 
    vtcFromButton = bvqx.FindChild("rbtnFromVTC"); 
    vmpFromButton = bvqx.FindChild("rbtnFromVMP");
    voiFromButton = bvqx.FindChild("rbtnFromVOI");
    niiToButton = bvqx.FindChild("rbtnToNII"); 
    hdrToButton = bvqx.FindChild("rbtnToHDR");
    vmrToButton = bvqx.FindChild("rbtnToVMR"); 
    fmrToButton = bvqx.FindChild("rbtnToFMR"); 
    dmrToButton = bvqx.FindChild("rbtnToDMR"); 
    vtcToButton = bvqx.FindChild("rbtnToVTC"); 
    vmpToButton = bvqx.FindChild("rbtnToVMP");
    transformBox = bvqx.FindChild("chbTrfImg");
    getBatchConvButton = bvqx.FindChild("btnBatchFiles");
    flipBox = bvqx.FindChild("chbFlip"); 
    batchBox = bvqx.FindChild("chbBatch"); 
    openinbvqxBox = bvqx.FindChild("chbOpenInBVQX"); 
    zeropadBox = bvqx.FindChild("chbZeropad");
    intensityBox = bvqx.FindChild("chbVMRintensity");
    vointerpBox = bvqx.FindChild("chboxVOIinterp");
    resetBox = bvqx.FindChild("chbReset");
    chbSelectVols = bvqx.FindChild("chbSelectVols");
    spbFromVol = bvqx.FindChild("spbFromVol"); 
    spbToVol = bvqx.FindChild("spbToVol");
    // information tab
    niiPrintButton = bvqx.FindChild("rbtnPrintNII"); 
    hdrPrintButton = bvqx.FindChild("rbtnPrintHDR");
    vmrPrintButton = bvqx.FindChild("rbtnPrintVMR"); 
    fmrPrintButton = bvqx.FindChild("rbtnPrintFMR"); 
    dmrPrintButton = bvqx.FindChild("rbtnPrintDMR"); 
    vtcPrintButton = bvqx.FindChild("rbtnPrintVTC"); 
    vmpPrintButton = bvqx.FindChild("rbtnPrintVMP");
    voiPrintButton = bvqx.FindChild("rbtnPrintVOI");
    destBothButton = bvqx.FindChild("rbtnDestBoth");
    destFileButton = bvqx.FindChild("rbtnDestFile"); 
    destLogButton = bvqx.FindChild("rbtnDestLog"); 
    // positioning tab
    getPosFileButton = bvqx.FindChild("btnGetPOSfile");
    createPosFileButton = bvqx.FindChild("rbtnCreatePosFile"); 
    posToImgButton = bvqx.FindChild("rbtnPosToImg"); 
    printPosMatButton = bvqx.FindChild("rbtnPrintMat"); 
    calcPosButton = bvqx.FindChild("rbtnComputePos");
    posFileLabel = bvqx.FindChild("lblPosFile");
    nxBox = bvqx.FindChild("nxBox");
    nyBox = bvqx.FindChild("nyBox");
    nzBox = bvqx.FindChild("nzBox");
    dxBox = bvqx.FindChild("dxBox");
    dyBox = bvqx.FindChild("dyBox");
    dzBox = bvqx.FindChild("dzBox");
    p00 = bvqx.FindChild("le00_p");
    p01 = bvqx.FindChild("le01_p");
    p02 = bvqx.FindChild("le02_p");
    p03 = bvqx.FindChild("le03_p");
    p10 = bvqx.FindChild("le10_p");
    p11 = bvqx.FindChild("le11_p");
    p12 = bvqx.FindChild("le12_p");
    p13 = bvqx.FindChild("le13_p");
    p20 = bvqx.FindChild("le20_p");
    p21 = bvqx.FindChild("le21_p");
    p22 = bvqx.FindChild("le22_p");
    p23 = bvqx.FindChild("le23_p");
    p30 = bvqx.FindChild("le30_p");
    p31 = bvqx.FindChild("le31_p");
    p32 = bvqx.FindChild("le32_p");
    p33 = bvqx.FindChild("le33_p");
    // header manipulation tab
    optCopyButton       = bvqx.FindChild("rbtnOptCopy");
    optFldManipButton   = bvqx.FindChild("rbtnOptManip");
    rbtnCopyPos         = bvqx.FindChild("rbtnCopyPos");
    rbtnCopyAll         = bvqx.FindChild("rbtnCopyAll");
    rbtnNiiType         = bvqx.FindChild("rbtnNiiType");
    rbtnHdrType         = bvqx.FindChild("rbtnHdrType");
    rbtnNiiTarget       = bvqx.FindChild("rbtnNiiTarget");
    rbtnHdrTarget       = bvqx.FindChild("rbtnHdrTarget");
    lblOtherNIfTIfile = bvqx.FindChild("lblOtherNIfTIfile");
    btnOtherNIfTIfile = bvqx.FindChild("btnOtherNIfTIfile");
    leQformCode = bvqx.FindChild("leQformCode");
    leQb = bvqx.FindChild("leQb");
    leQc = bvqx.FindChild("leQc");
    leQd = bvqx.FindChild("leQd");
    leQoffsetX = bvqx.FindChild("leQoffsetX"); 
    leQoffsetY = bvqx.FindChild("leQoffsetY");
    leQoffsetZ = bvqx.FindChild("leQoffsetZ");
    leSformCode = bvqx.FindChild("leSformCode");
    leSrow00 = bvqx.FindChild("leSrow00");
    leSrow01 = bvqx.FindChild("leSrow01");
    leSrow02 = bvqx.FindChild("leSrow02");
    leSrow03 = bvqx.FindChild("leSrow03");
    leSrow10 = bvqx.FindChild("leSrow10");
    leSrow11 = bvqx.FindChild("leSrow11");
    leSrow12 = bvqx.FindChild("leSrow12");
    leSrow13 = bvqx.FindChild("leSrow13");    
    leSrow20 = bvqx.FindChild("leSrow20");
    leSrow21 = bvqx.FindChild("leSrow21");
    leSrow22 = bvqx.FindChild("leSrow22");
    leSrow23 = bvqx.FindChild("leSrow23");
    leSrow30 = bvqx.FindChild("leSrow30"); // fixed 19-09-11
    leSrow31 = bvqx.FindChild("leSrow31"); // fixed 19-09-11
    leSrow32 = bvqx.FindChild("leSrow32"); // fixed 19-09-11
    leSrow33 = bvqx.FindChild("leSrow33"); // fixed 19-09-11
    calcBVposButton = bvqx.FindChild("btnCalcBVpos");
    leQFac = bvqx.FindChild("leQFac");
    lblNx = bvqx.FindChild("lblNx");
    lblNy = bvqx.FindChild("lblNy");
    lblNz = bvqx.FindChild("lblNz");
    lblDx = bvqx.FindChild("lblDx");
    lblDy = bvqx.FindChild("lblDy");
    lblDz = bvqx.FindChild("lblDz");
    
    // transformation tab
    invertTrfButton         = bvqx.FindChild("rbtnInvertTrf");
    applyManTrfButton       = bvqx.FindChild("rbtnApplyManTrf");
    applyBVQXTrfButton      = bvqx.FindChild("rbtnApplyBVQXtrf");
    getTrfFileTypeBox       = bvqx.FindChild("cmBoxTrfFiles");
    transformGrBox          = bvqx.FindChild("grboxTrfMat");
    getTrfFileButton        = bvqx.FindChild("btnGetTrfFile");
    trfFileLabel            = bvqx.FindChild("lblTrfFile");
    comboboxListenerButton  = bvqx.FindChild("btnCmbListener");
    t00 = bvqx.FindChild("le00"); 
    t01 = bvqx.FindChild("le01"); 
    t02 = bvqx.FindChild("le02"); 
    t03 = bvqx.FindChild("le03"); 
    t10 = bvqx.FindChild("le10"); 
    t11 = bvqx.FindChild("le11"); 
    t12 = bvqx.FindChild("le12"); 
    t13 = bvqx.FindChild("le13"); 
    t20 = bvqx.FindChild("le20"); 
    t21 = bvqx.FindChild("le21"); 
    t22 = bvqx.FindChild("le22"); 
    t23 = bvqx.FindChild("le23"); 
    t30 = bvqx.FindChild("le30"); 
    t31 = bvqx.FindChild("le31"); 
    t32 = bvqx.FindChild("le32"); 
    t33 = bvqx.FindChild("le33");     
    // batch tab
    writeBatchFileButton = bvqx.FindChild("rbtnWriteBatch");

    // connect widgets to functions here
    getBatchConvButton.clicked.connect(this, this.getBatchFile);    
    getNiftiFileButton.clicked.connect(this, this.getNiftiFile);
    btnOtherNIfTIfile.clicked.connect(this, this.getOtherNiftiFile);
    getPosFileButton.clicked.connect(this, this.getPosFile);    
    getTrfFileButton.clicked.connect(this, this.getTrfFile);
    comboboxListenerButton.clicked.connect(this, this.checkTrfFileTypeBox);
    calcBVposButton.clicked.connect(this, this.calculateBrainVoyagerPositionFromNIfTI); 
    startButton.clicked.connect(this, this.start);

	// set property
 	comboboxListenerButton.visible = false;
   
}

returnScriptObj = function() { 
	return scriptObj; 
} 

returnScriptObj(); // should be after returnScriptOjb = function()
